@@ -39,6 +39,7 @@
 #include "pygi-closure.h"
 #include "pygi-type.h"
 #include "pygi-boxed.h"
+#include "pygi-fundamental.h"
 #include "pygi-info.h"
 #include "pygi-struct.h"
 #include "pygobject-object.h"
@@ -54,15 +55,12 @@
 #include "gimodule.h"
 #include "pygi-basictype.h"
 
-/* GI_CHECK_VERSION was introduced in gobject-introspection 1.60 */
-#ifndef GI_CHECK_VERSION
-# define GI_CHECK_VERSION(x,y,z) 0
-#endif
-
 PyObject *PyGIWarning;
 PyObject *PyGIDeprecationWarning;
 PyObject *_PyGIDefaultArgPlaceholder;
 
+static int _gi_exec (PyObject *module);
+
 /* Returns a new flag/enum type or %NULL */
 static PyObject *
 flags_enum_from_gtype (GType g_type,
@@ -1279,16 +1277,21 @@ pyg_type_register(PyTypeObject *class, const char *type_name)
 	(GBaseInitFunc) NULL,
 	(GBaseFinalizeFunc) NULL,
 
-	(GClassInitFunc) pyg_object_class_init,
+	(GClassInitFunc) NULL,
 	(GClassFinalizeFunc) NULL,
 	NULL, /* class_data */
 
 	0,    /* instance_size */
 	0,    /* n_preallocs */
-	(GInstanceInitFunc) pygobject__g_instance_init
+	(GInstanceInitFunc) NULL
     };
     gchar *new_type_name;
 
+    if (PyType_IsSubtype(class, &PyGObject_Type)) {
+        type_info.class_init = (GClassInitFunc) pyg_object_class_init;
+        type_info.instance_init = pygobject__g_instance_init;
+    }
+
     /* find the GType of the parent */
     parent_type = pyg_type_from_object((PyObject *)class);
     if (!parent_type)
@@ -1387,15 +1390,17 @@ _wrap_pyg_type_register(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "O!|z:gobject.type_register",
 			  &PyType_Type, &class, &type_name))
 	return NULL;
-    if (!PyType_IsSubtype(class, &PyGObject_Type)) {
+
+    GType base_gtype = pyg_type_from_object((PyObject *) class->tp_base);
+
+    if (base_gtype == G_TYPE_INVALID) {
 	PyErr_SetString(PyExc_TypeError,
-			"argument must be a GObject subclass");
+			"argument must be a Fundamental or GObject subclass");
 	return NULL;
     }
 
       /* Check if type already registered */
-    if (pyg_type_from_object((PyObject *) class) ==
-        pyg_type_from_object((PyObject *) class->tp_base))
+    if (pyg_type_from_object((PyObject *) class) == base_gtype)
     {
         if (pyg_type_register(class, type_name))
             return NULL;
@@ -2519,13 +2524,18 @@ pygi_register_version_tuples(PyObject *d)
     return 0;
 }
 
+static PyModuleDef_Slot _gi_slots[] = {
+    {Py_mod_exec, _gi_exec},
+    {0, NULL}
+};
+
 static struct PyModuleDef __gimodule = {
     PyModuleDef_HEAD_INIT,
     "_gi",
     NULL,
-    -1,
+    0,
     _gi_functions,
-    NULL,
+    _gi_slots,
     NULL,
     NULL,
     NULL
@@ -2540,10 +2550,15 @@ static struct PyModuleDef __gimodule = {
 PYGI_MODINIT_FUNC PyInit__gi(void);
 
 PYGI_MODINIT_FUNC PyInit__gi(void) {
-    PyObject *module;
+    return PyModuleDef_Init(&__gimodule);
+}
+
+static int
+_gi_exec (PyObject *module)
+{
     PyObject *api;
-    module = PyModule_Create(&__gimodule);
     PyObject *module_dict = PyModule_GetDict (module);
+    int ret;
 
 #if PY_VERSION_HEX < 0x03090000 || defined(PYPY_VERSION)
     /* Deprecated since 3.9 */
@@ -2559,58 +2574,60 @@ PYGI_MODINIT_FUNC PyInit__gi(void) {
 
     PyModule_AddStringConstant(module, "__package__", "gi._gi");
 
-    if (pygi_foreign_init () < 0)
-        return NULL;
-    if (pygi_error_register_types (module) < 0)
-        return NULL;
-    if (pygi_repository_register_types (module) < 0)
-        return NULL;
-    if (pygi_info_register_types (module) < 0)
-        return NULL;
-    if (pygi_type_register_types (module_dict) < 0)
-        return NULL;
-    if (pygi_pointer_register_types (module_dict) < 0)
-        return NULL;
-    if (pygi_struct_register_types (module) < 0)
-        return NULL;
-    if (pygi_gboxed_register_types (module_dict) < 0)
-        return NULL;
-    if (pygi_boxed_register_types (module) < 0)
-        return NULL;
-    if (pygi_ccallback_register_types (module) < 0)
-        return NULL;
-    if (pygi_resulttuple_register_types (module) < 0)
-        return NULL;
+    if ((ret = pygi_foreign_init ()) < 0)
+        return ret;
+    if ((ret = pygi_error_register_types (module)) < 0)
+        return ret;
+    if ((ret = pygi_repository_register_types (module)) < 0)
+        return ret;
+    if ((ret = pygi_info_register_types (module)) < 0)
+        return ret;
+    if ((ret = pygi_type_register_types (module_dict)) < 0)
+        return ret;
+    if ((ret = pygi_pointer_register_types (module_dict)) < 0)
+        return ret;
+    if ((ret = pygi_struct_register_types (module)) < 0)
+        return ret;
+    if ((ret = pygi_gboxed_register_types (module_dict)) < 0)
+        return ret;
+    if ((ret = pygi_fundamental_register_types (module)) < 0)
+        return ret;
+    if ((ret = pygi_boxed_register_types (module)) < 0)
+        return ret;
+    if ((ret = pygi_ccallback_register_types (module)) < 0)
+        return ret;
+    if ((ret = pygi_resulttuple_register_types (module)) < 0)
+        return ret;
 
-    if (pygi_spawn_register_types (module_dict) < 0)
-        return NULL;
-    if (pygi_option_context_register_types (module_dict) < 0)
-        return NULL;
-    if (pygi_option_group_register_types (module_dict) < 0)
-        return NULL;
+    if ((ret = pygi_spawn_register_types (module_dict)) < 0)
+        return ret;
+    if ((ret = pygi_option_context_register_types (module_dict)) < 0)
+        return ret;
+    if ((ret = pygi_option_group_register_types (module_dict)) < 0)
+        return ret;
 
-    if (pygi_register_api (module_dict) < 0)
-        return NULL;
-    if (pygi_register_constants (module) < 0)
-        return NULL;
-    if (pygi_register_version_tuples (module_dict) < 0)
-        return NULL;
-    if (pygi_register_warnings (module_dict) < 0)
-        return NULL;
-    if (pyi_object_register_types (module_dict) < 0)
-        return NULL;
-    if (pygi_interface_register_types (module_dict) < 0)
-        return NULL;
-    if (pygi_paramspec_register_types (module_dict) < 0)
-        return NULL;
-    if (pygi_enum_register_types (module_dict) < 0)
-        return NULL;
-    if (pygi_flags_register_types (module_dict) < 0)
-        return NULL;
+    if ((ret = pygi_register_api (module_dict)) < 0)
+        return ret;
+    if ((ret = pygi_register_constants (module)) < 0)
+        return ret;
+    if ((ret = pygi_register_version_tuples (module_dict)) < 0)
+        return ret;
+    if ((ret = pygi_register_warnings (module_dict)) < 0)
+        return ret;
+    if ((ret = pyi_object_register_types (module_dict)) < 0)
+        return ret;
+    if ((ret = pygi_interface_register_types (module_dict)) < 0)
+        return ret;
+    if ((ret = pygi_paramspec_register_types (module_dict)) < 0)
+        return ret;
+    if ((ret = pygi_enum_register_types (module_dict)) < 0)
+        return ret;
+    if ((ret = pygi_flags_register_types (module_dict)) < 0)
+        return ret;
 
     PyGIWarning = PyErr_NewException ("gi.PyGIWarning", PyExc_Warning, NULL);
     if (PyGIWarning == NULL)
-        return NULL;
+        return -1;
 
     PyGIDeprecationWarning = PyErr_NewException("gi.PyGIDeprecationWarning",
                                                 PyExc_DeprecationWarning, NULL);
@@ -2628,9 +2645,9 @@ PYGI_MODINIT_FUNC PyInit__gi(void) {
 
     api = PyCapsule_New ( (void *) &CAPI, "gi._API", NULL);
     if (api == NULL) {
-        return NULL;
+        return -1;
     }
     PyModule_AddObject (module, "_API", api);
 
-    return module;
+    return 0;
 }
