@@ -15,9 +15,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
- * USA
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __PYGOBJECT_PARAMSPEC_H__ 
@@ -28,6 +26,6 @@
 extern PyTypeObject PyGParamSpec_Type;
 PyObject * pyg_param_spec_new (GParamSpec *pspec);
 
-void pygobject_paramspec_register_types(PyObject *d);
+int pygi_paramspec_register_types(PyObject *d);
 
 #endif /* __PYGOBJECT_PARAMSPEC_H__ */
